/**
 * This class was created by BrassGoggledCoders modding team.
 * This class is available as part of the BoilerCraft Mod for Minecraft.
 *
 * BoilerCraft is open-source and is distributed under the MMPL v1.0 License.
 * (http://www.mod-buildcraft.com/MMPL-1.0.txt)
 *
 */
package xyz.brassgoggledcoders.boilerplate.client.utils;

import xyz.brassgoggledcoders.boilerplate.client.ClientHelper;
import net.minecraft.client.particle.EntityFX;
import net.minecraft.client.particle.EntityFlameFX;
import net.minecraft.client.particle.EntitySmokeFX;
import net.minecraft.util.BlockPos;
import net.minecraft.util.EnumParticleTypes;
import net.minecraft.world.World;
import org.lwjgl.opengl.GL11;

import java.util.Random;

/**
 * @author Surseance
 *
 */
public class EffectUtils
{
	private static EntityFlameFX.Factory flameFactory;
	private static EntitySmokeFX.Factory smokeFactory;

	public static EntityFlameFX.Factory getFlameFactory()
	{
		if(flameFactory == null)
		{
			flameFactory = new EntityFlameFX.Factory();
		}

		return flameFactory;
	}

	public static EntitySmokeFX.Factory getSmokeFactory()
	{
		if(smokeFactory == null)
		{
			smokeFactory = new EntitySmokeFX.Factory();
		}

		return smokeFactory;
	}

	public static void sparkle(final World world, final BlockPos blockPos, final EnumParticleTypes particleType)
	{
		final Random random = world.field_73012_v;
		final double offset = 0.0625D;

		for (int amount = 0; amount < 6; amount++)
		{
			double dx = blockPos.func_177958_n() + random.nextFloat();
			double dy = blockPos.func_177956_o() + random.nextFloat();
			double dz = blockPos.func_177952_p() + random.nextFloat();

			if ((amount == 0) && (!world.func_175677_d(blockPos.func_177984_a(), false)))
			{
				dy = blockPos.func_177956_o() + 1 + offset;
			}

			if ((amount == 1) && (!world.func_175677_d(blockPos.func_177977_b(), false)))
			{
				dy = blockPos.func_177956_o() - offset;
			}

			if ((amount == 2) && (!world.func_175677_d(blockPos.func_177968_d(), false)))
			{
				dz = blockPos.func_177952_p() + 1 + offset;
			}

			if ((amount == 3) && (!world.func_175677_d(blockPos.func_177978_c(), false)))
			{
				dz = blockPos.func_177956_o() - offset;
			}

			if ((amount == 4) && (!world.func_175677_d(blockPos.func_177974_f(), false)))
			{
				dx = blockPos.func_177958_n() + 1 + offset;
			}

			if ((amount == 5) && (!world.func_175677_d(blockPos.func_177976_e(), false)))
			{
				dx = blockPos.func_177958_n() - offset;
			}

			if ((dx < blockPos.func_177958_n()) || (dx > (blockPos.func_177958_n() + 1)) || (dy < 0.0D) ||
					(dy > (blockPos.func_177956_o() + 1)) || (dz < blockPos.func_177952_p()) || (dz > (blockPos.func_177952_p() + 1)))
			{
				world.func_175688_a(particleType, dx, dy, dz, -1.0D, 1.0D, -1.0D);
			}
		}
	}

	public static void blockSparkle(World world, int x, int y, int z, int count)
	{
		if (!world.field_72995_K)
		{
			return;
		}

		for (; count < 10; ++count)
		{
			double startX = x + world.field_73012_v.nextFloat();
			double startY = y + (world.field_73012_v.nextFloat() * 1.0F);
			double startZ = z + world.field_73012_v.nextFloat();

			double endX = world.field_73012_v.nextGaussian() * 0.02D;
			double endY = world.field_73012_v.nextGaussian() * 0.02D;
			double endZ = world.field_73012_v.nextGaussian() * 0.02D;

			EntityFlameFX ef = getEntityFlameFX(world, startX, startY, startZ, endX, endY, endZ);
			ef.func_70538_b(0.45F, 0.0F, 0.35F);
			ef.func_70536_a(82);
			ClientHelper.effectRenderer().func_78873_a(ef);
		}
	}

	public static void blockFlameFX(World world, int x, int y, int z, int count)
	{
		for (; count < 10; ++count)
		{
			double startX = x + world.field_73012_v.nextFloat();
			double startY = y + (world.field_73012_v.nextFloat() * 1.0F);
			double startZ = z + world.field_73012_v.nextFloat();

			double endX = world.field_73012_v.nextGaussian() * 0.02D;
			double endY = world.field_73012_v.nextGaussian() * 0.02D;
			double endZ = world.field_73012_v.nextGaussian() * 0.02D;

			EntityFlameFX ef = getEntityFlameFX(world, startX, startY, startZ, endX, endY, endZ);
			ClientHelper.effectRenderer().func_78873_a(ef);
		}
	}

	private static EntityFX particle;

	public static void displayFX(final String name, final World world, final double dx, final double dy, final double dz, final double velX,
			final double velY, final double velZ, final float scale)
	{
		particle = null;

		/** An example of adding an EntityFX class. */
		if (name.equals("smoke"))
		{
			particle = getEntitySmokeFX(world, dx, dy, dz, velX, velY, velZ);
		}


		final double distX = ClientHelper.viewEntity().field_70165_t - particle.field_70165_t;
		final double distY = ClientHelper.viewEntity().field_70163_u - particle.field_70163_u;
		final double distZ = ClientHelper.viewEntity().field_70161_v - particle.field_70161_v;
		int display = ClientHelper.settings().field_74362_aa;

		if ((display == 1) && (particle.field_70170_p.field_73012_v.nextInt(3) == 0))
		{
			display = 2;
		}
		if ((display <= 1) && (((distX * distX) + (distY * distY) + (distZ * distZ)) <= 4096.0D))
		{
			ClientHelper.effectRenderer().func_78873_a(particle);
		}
	}

	public static EntityFlameFX getEntityFlameFX(World worldIn, double xPos, double yPos, double zPox, double xVec, double yVec, double zVec)
	{
		return (EntityFlameFX)getFlameFactory().func_178902_a(0, worldIn, xPos, yPos, zPox, xVec, yVec, zVec);
	}

	public static EntitySmokeFX getEntitySmokeFX(World worldIn, double xPos, double yPos, double zPox, double xVec, double yVec, double zVec)
	{
		return (EntitySmokeFX)getSmokeFactory().func_178902_a(0, worldIn, xPos, yPos, zPox, xVec, yVec, zVec);
	}

	public static void setGLColorFromInt(int color)
	{
		float red = ((color >> 16) & 255) / 255.0F;
		float green = ((color >> 8) & 255) / 255.0F;
		float blue = (color & 255) / 255.0F;
		GL11.glColor4f(red, green, blue, 1.0F);
	}
}
